@@ -10,7 +10,6 @@ use options::*;
 use writer::*;
 
 pub use self::LabelType::*;
-pub use self::PlotType::*;
 pub use self::TickAxis::*;
 pub use self::PlotType::*;
 pub use self::DataSourceType::*;
@@ -72,11 +71,13 @@ impl PlotElement
 	}
 }
 
+#[deriving(Copy)]
 pub enum LabelType
 {
 	XLabel,
 	YLabel,
 	ZLabel,
+	CBLabel,
 	TitleLabel,
 	Label(Coordinate, Coordinate),
 	AxesTicks,
@@ -107,63 +108,63 @@ pub fn write_out_label_options<T: PlotWriter + Writer>(label_type: LabelType, op
 		_ => ()
 	}
 
-	first_opt!(options,
+	first_opt!{options,
 		TextOffset(x, y) =>
 		{
 			write!(w, " offset character {:.12e},{:.12e}", x, y);
 		}
-	)
+	}
 
-	first_opt!(options,
+	first_opt!{options,
 		TextColor(s) =>
 		{
 			write!(w, r#" tc rgb "{}""#, s);
 		}
-	)
+	}
 
-	first_opt!(options,
+	first_opt!{options,
 		Font(f, s) =>
 		{
 			write!(w, r#" font "{},{}""#, f, s);
 		}
-	)
+	}
 
-	first_opt!(options,
+	first_opt!{options,
 		Rotate(a) =>
 		{
 			write!(w, " rotate by {:.12e}", a);
 		}
-	)
+	}
 
 	if label_type.is_label()
 	{
 		let mut have_point = false;
-		first_opt!(options,
+		first_opt!{options,
 			MarkerSymbol(s) =>
 			{
 				write!(w, " point pt {}", char_to_symbol(s));
 				have_point = true;
 			}
-		)
+		}
 
 		if have_point
 		{
-			first_opt!(options,
+			first_opt!{options,
 				MarkerColor(s) =>
 				{
 					write!(w, r#" lc rgb "{}""#, s);
 				}
-			)
+			}
 
-			first_opt!(options,
+			first_opt!{options,
 				MarkerSize(z) =>
 				{
 					write!(w, " ps {:.12e}", z);
 				}
-			)
+			}
 		}
 
-		first_opt!(options,
+		first_opt!{options,
 			TextAlign(a) =>
 			{
 				write!(w, "{}", match a
@@ -173,7 +174,7 @@ pub fn write_out_label_options<T: PlotWriter + Writer>(label_type: LabelType, op
 					_ => " center",
 				});
 			}
-		)
+		}
 	}
 }
 
@@ -182,6 +183,7 @@ pub enum TickAxis
 	XTickAxis,
 	YTickAxis,
 	ZTickAxis,
+	CBTickAxis,
 }
 
 impl TickAxis
@@ -193,6 +195,7 @@ impl TickAxis
 			XTickAxis => "x",
 			YTickAxis => "y",
 			ZTickAxis => "z",
+			CBTickAxis => "cb",
 		}
 	}
 
@@ -203,6 +206,7 @@ impl TickAxis
 			XTickAxis => "xtics",
 			YTickAxis => "ytics",
 			ZTickAxis => "ztics",
+			CBTickAxis => "cbtics",
 		}
 	}
 
@@ -213,6 +217,7 @@ impl TickAxis
 			XTickAxis => "xrange",
 			YTickAxis => "yrange",
 			ZTickAxis => "zrange",
+			CBTickAxis => "cbrange",
 		}
 	}
 }
@@ -410,7 +415,7 @@ impl AxisData
 		let c = &mut self.ticks_buf;
 		write_out_label_options(AxesTicks, label_options, c);
 
-		first_opt!(tick_options,
+		first_opt!{tick_options,
 			OnAxis(b) =>
 			{
 				c.write_str(match b
@@ -419,9 +424,9 @@ impl AxisData
 					false => " border",
 				});
 			}
-		)
+		}
 
-		first_opt!(tick_options,
+		first_opt!{tick_options,
 			Mirror(b) =>
 			{
 				c.write_str(match b
@@ -430,9 +435,9 @@ impl AxisData
 					false => " nomirror",
 				});
 			}
-		)
+		}
 
-		first_opt!(tick_options,
+		first_opt!{tick_options,
 			Inward(b) =>
 			{
 				c.write_str(match b
@@ -441,24 +446,24 @@ impl AxisData
 					false => " out",
 				});
 			}
-		)
+		}
 
 		let mut minor_scale = 0.5;
 		let mut major_scale = 0.5;
 
-		first_opt!(tick_options,
+		first_opt!{tick_options,
 			MinorScale(s) =>
 			{
 				minor_scale = s;
 			}
-		)
+		}
 
-		first_opt!(tick_options,
+		first_opt!{tick_options,
 			MajorScale(s) =>
 			{
 				major_scale = s;
 			}
-		)
+		}
 
 		write!(&mut *c, " scale {:.12e},{:.12e}", minor_scale, major_scale);
 	}
@@ -527,6 +532,7 @@ pub struct AxesCommonData
 	pub grid_pos: Option<u32>,
 	pub x_axis: AxisData,
 	pub y_axis: AxisData,
+	pub cb_axis: AxisData,
 }
 
 pub fn char_to_symbol(c: char) -> i32
@@ -571,6 +577,7 @@ impl AxesCommonData
 			grid_pos: None,
 			x_axis: AxisData::new(XTickAxis),
 			y_axis: AxisData::new(YTickAxis),
+			cb_axis: AxisData::new(CBTickAxis),
 		}
 	}
 
@@ -578,13 +585,13 @@ impl AxesCommonData
 	{
 		let mut found = false;
 		c.write_str(" lw ");
-		first_opt!(options,
+		first_opt!{options,
 			LineWidth(w) =>
 			{
 				write!(c, "{:.12e}", w);
 				found = true;
 			}
-		)
+		}
 		if !found
 		{
 			c.write_str("1");
@@ -592,13 +599,13 @@ impl AxesCommonData
 
 		c.write_str(" lt ");
 		let mut found = false;
-		first_opt!(options,
+		first_opt!{options,
 			LineStyle(d) =>
 			{
 				write!(c, "{}", d.to_int());
 				found = true;
 			}
-		)
+		}
 		if !found
 		{
 			c.write_str("1");
@@ -608,12 +615,12 @@ impl AxesCommonData
 	pub fn write_color_options<'l>(c: &mut Writer, options: &[PlotOption<'l>], default: Option<&'l str>)
 	{
 		let mut col = default;
-		first_opt!(options,
+		first_opt!{options,
 			Color(s) =>
 			{
 				col = Some(s)
 			}
-		)
+		}
 		match col
 		{
 			Some(s) =>
@@ -795,7 +802,7 @@ impl AxesCommonData
 				FillBetween =>
 				{
 					let mut found = false;
-					first_opt!(options,
+					first_opt!{options,
 						FillRegion(d) =>
 						{
 							found = true;
@@ -806,7 +813,7 @@ impl AxesCommonData
 								Between => " closed",
 							});
 						}
-					)
+					}
 					if !found
 					{
 						args.write_str(" closed");
@@ -817,22 +824,22 @@ impl AxesCommonData
 
 			args.write_str(" fill transparent solid ");
 
-			first_opt!(options,
+			first_opt!{options,
 				FillAlpha(a) =>
 				{
 					write!(args, "{:.12e}", a);
 				}
-			)
+			}
 
 			if plot_type.is_line()
 			{
 				args.write_str(" border");
-				first_opt!(options,
+				first_opt!{options,
 					BorderColor(s) =>
 					{
 						write!(args, r#" rgb "{}""#, s);
 					}
-				)
+				}
 			}
 			else
 			{
@@ -847,30 +854,30 @@ impl AxesCommonData
 
 		if plot_type.is_points()
 		{
-			first_opt!(options,
+			first_opt!{options,
 				PointSymbol(s) =>
 				{
 					write!(args, " pt {}", char_to_symbol(s));
 				}
-			)
+			}
 
-			first_opt!(options,
+			first_opt!{options,
 				PointSize(z) =>
 				{
 					write!(args, " ps {}", z);
 				}
-			)
+			}
 		}
 
 		AxesCommonData::write_color_options(args, options, None);
 
 		args.write_str(" t \"");
-		first_opt!(options,
+		first_opt!{options,
 			Caption(s) =>
 			{
 				args.write_str(s);
 			}
-		)
+		}
 		args.write_str("\"");
 	}
 
@@ -879,6 +886,7 @@ impl AxesCommonData
 		writer.write(self.commands.get_ref());
 		self.x_axis.write_out_commands(writer);
 		self.y_axis.write_out_commands(writer);
+		self.cb_axis.write_out_commands(writer);
 	}
 
 	pub fn write_out_elements(&self, cmd: &str, writer: &mut Writer)
@@ -915,6 +923,7 @@ impl AxesCommonData
 			XLabel => "xlabel",
 			YLabel => "ylabel",
 			ZLabel => "zlabel",
+			CBLabel => "cblabe",
 			TitleLabel => "title",
 			Label(..) => "label",
 			_ => panic!("Invalid label type")
@@ -1020,21 +1029,20 @@ pub trait AxesCommon : AxesCommonPrivate
 		self
 	}
 
-	/// Set the label for the Y axis
-	/// # Arguments
-	/// * `text` - Text of the label. Pass an empty string to hide the label
-	/// * `options` - Array of LabelOption controlling the appearance of the label. Relevant options are:
-	///      * `Offset` - Specifies the offset of the label
-	///      * `Font` - Specifies the font of the label
-	///      * `TextColor` - Specifies the color of the label
-	///      * `Rotate` - Specifies the rotation of the label
-	///      * `Align` - Specifies how to align the label
+	/// Like `set_x_label`, but for the Y axis
 	fn set_y_label<'l>(&'l mut self, text: &str, options: &[LabelOption]) -> &'l mut Self
 	{
 		self.get_common_data_mut().set_label_common(YLabel, text, options);
 		self
 	}
 
+	/// Like `set_x_label`, but for the color bar
+	fn set_cb_label<'l>(&'l mut self, text: &str, options: &[LabelOption]) -> &'l mut Self
+	{
+		self.get_common_data_mut().set_label_common(CBLabel, text, options);
+		self
+	}
+
 	/// Set the title for the axes
 	/// # Arguments
 	/// * `text` - Text of the title. Pass an empty string to hide the title
@@ -1097,6 +1105,13 @@ pub trait AxesCommon : AxesCommonPrivate
 		self
 	}
 
+	/// Like `set_x_ticks` but for the color bar axis.
+	fn set_cb_ticks<'l>(&'l mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Self
+	{
+		self.get_common_data_mut().cb_axis.set_ticks(tick_placement, tick_options, label_options);
+		self
+	}
+
 	/// Sets ticks on the X axis with specified labels at specified positions.
 	///
 	/// # Arguments
@@ -1124,6 +1139,13 @@ pub trait AxesCommon : AxesCommonPrivate
 		self
 	}
 
+	/// Like `set_x_ticks_custom` but for the the color bar axis.
+	fn set_cb_ticks_custom<'l, T: DataType, TL: Iterator<Tick<T>>>(&'l mut self, ticks: TL, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Self
+	{
+		self.get_common_data_mut().cb_axis.set_ticks_custom(ticks, tick_options, label_options);
+		self
+	}
+
 	/// Set the range of values for the X axis.
 	///
 	/// # Arguments
@@ -1146,6 +1168,17 @@ pub trait AxesCommon : AxesCommonPrivate
 		self
 	}
 
+	/// Set the range of values for the color bar axis.
+	///
+	/// # Arguments
+	/// * `min` - Minimum Y value
+	/// * `max` - Maximum Y value
+	fn set_cb_range<'l>(&'l mut self, min: AutoOption<f64>, max: AutoOption<f64>) -> &'l mut Self
+	{
+		self.get_common_data_mut().cb_axis.set_range(min, max);
+		self
+	}
+
 	/// Sets the X axis be logarithmic. Note that the range must be non-negative for this to be valid.
 	///
 	/// # Arguments
@@ -1165,4 +1198,91 @@ pub trait AxesCommon : AxesCommonPrivate
 		self.get_common_data_mut().y_axis.set_log(base);
 		self
 	}
+
+	/// Sets the color bar axis be logarithmic. Note that the range must be non-negative for this to be valid.
+	///
+	/// # Arguments
+	/// * `base` - If Some, then specifies base of the logarithm, if None makes the axis not be logarithmic
+	fn set_cb_log<'l>(&'l mut self, base: Option<f64>) -> &'l mut Self
+	{
+		self.get_common_data_mut().cb_axis.set_log(base);
+		self
+	}
+
+	/// Sets the palette used for 3D surface and image plots
+	///
+	/// # Arguments
+	/// * `palette` - What palette type to use
+	fn set_palette(&mut self, palette: PaletteType) -> &mut Self
+	{
+		{
+			let c = &mut self.get_common_data_mut().commands as &mut Writer;
+			match palette
+			{
+				Gray(gamma) =>
+				{
+					assert!(gamma > 0.0, "Gamma must be positive");
+					writeln!(c, "set palette gray gamma {:.12e}", gamma);
+				},
+				Formula(r, g, b) =>
+				{
+					assert!(r >= -36 && r <= 36, "Invalid r formula!");
+					assert!(g >= -36 && g <= 36, "Invalid g formula!");
+					assert!(b >= -36 && b <= 36, "Invalid b formula!");
+					writeln!(c, "set palette rgbformulae {},{},{}", r, g, b);
+				},
+				CubeHelix(start, rev, sat, gamma) =>
+				{
+					assert!(sat >= 0.0, "Saturation must be non-negative");
+					assert!(gamma > 0.0, "Gamma must be positive");
+					writeln!(c, "set palette cubehelix start {:.12e} cycles {:.12e} saturation {:.12e} gamma {:.12e}", start, rev, sat, gamma);
+				},
+			}
+		}
+		self
+	}
+
+	/// Sets a custom palette used for 3D surface and image plots. A custom palette
+	/// is specified by a sequence of 4-tuples (with at least one element). The first
+	/// element is the grayscale value that is mapped to the remaining three elements
+	/// which specify the red, green and blue components of the color.
+	/// The grayscale values must be non-decreasing. All values must range from 0 to 1.
+	///
+	/// # Arguments
+	/// * `palette_generator` - The palette generator
+	fn set_custom_palette<T: Iterator<(f32, f32, f32, f32)>>(&mut self, palette_generator: T) -> &mut Self
+	{
+		let mut palette_generator = palette_generator;
+		{
+			let c = &mut self.get_common_data_mut().commands as &mut Writer;
+			write!(c, "set palette defined (");
+
+			let mut first = true;
+			let mut old_x = 0.0;
+			for (x, r, g, b) in palette_generator
+			{
+				if first
+				{
+					old_x = x;
+					first = false;
+				}
+				else
+				{
+					write!(c, ",");
+				}
+				assert!(x >= old_x, "The gray levels must be non-decreasing!");
+				old_x = x;
+
+				write!(c, "{:.12e} {:.12e} {:.12e} {:.12e}", x, r, g, b);
+			}
+
+			if first
+			{
+				panic!("Need at least 1 element in the generator");
+			}
+
+			writeln!(c, ")");
+		}
+		self
+	}
 }
